/* 
 * File: ChatClient.java
 *
 */

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;

import java.io.*;
import java.net.*;

public class ChatClient extends Applet implements ActionListener {
  private Button button;
  
  public void init() {
    button = new Button( "Chat Board" );
    add( button );
    button.addActionListener( this );    
  }
  
  public void actionPerformed( ActionEvent e ) {
    new ChatWindow(this);
  }
  
  // This applet also runs as an application:
  public static void main( String[] args ) {
    new ChatWindow(new ChatClient());
  }
  
}

class ChatWindow extends Frame {
  ChatClient client;

  public static final int PORT = 2222;
  Socket s;
  DataInputStream in;
  PrintStream out;
  StreamListener listener;
  String inputString = new String("anonymous");

  StringBuffer uname, rname;  
  protected static int num_windows = 0;     // number of open windows
  
  // GUI components:
  MenuBar menubar;
  ConnectMenu cMenu;
  RoomMenu rMenu;

  TextArea textarea;
  TextField inputfield;
  
  // Constructor:
  public ChatWindow(ChatClient client) {  
    // Invoke the constructor of the superclass:
    super( "Chat board" );
    this.client=client;
    
    num_windows++;                          // update window counter
    
    // Set frame parameters:
    setSize( 700, 500 );
    setFont( new Font( "SansSerif", Font.BOLD, 12 ) );
    
    textarea = new TextArea();
    add( textarea, BorderLayout.CENTER );
    textarea.setFont( new Font( "TimesRoman", Font.PLAIN,12 ) );
    textarea.setEditable(false);

    inputfield = new TextField();
    //    inputfield.setFont( new Font ("Monospaced", Font.PLAIN, 10 ));
    add(inputfield, BorderLayout.SOUTH);
    
    menubar = new MenuBar();
    menubar.add( cMenu = new ConnectMenu() );
    menubar.add( rMenu = new RoomMenu() );

    menubar.setHelpMenu( new HelpMenu() );  // the Help menu is special
    
    setMenuBar( menubar );
    setVisible( true );

    //Establish connection
    try {
      s = new Socket(client.getCodeBase().getHost(), PORT);
      in = new DataInputStream(s.getInputStream());
      out = new PrintStream(s.getOutputStream());

      listener = new StreamListener(in, textarea);

      client.showStatus("Connected to "
                      + s.getInetAddress().getHostName()
                      + ":" + s.getPort());
    }
    catch (IOException e) { 
      client.showStatus(e.toString()); 
    }

    // all components can generate WindowEvents
    addWindowListener (new CloseWindow());
  }

  class ConnectMenu extends Menu implements ActionListener {
    // Menu item names:
    MenuItem connectItem, listItem, exitItem;
    
    // Menu constructor:
    public ConnectMenu() {
      // Invoke the constructor of the superclass:
      super( "Connection" );

      // Menu items with shortcuts:
      connectItem  
        = new MenuItem( "Connect as ...", 
                        new MenuShortcut( KeyEvent.VK_C ) );
      listItem 
        = new MenuItem( "List users", 
                        new MenuShortcut( KeyEvent.VK_U ) );
      exitItem           
        = new MenuItem( "Exit", 
                        new MenuShortcut( KeyEvent.VK_Q ) );
      
      // Build the menu:
      add( connectItem );
      add( listItem );
      addSeparator();      
      add( exitItem );
      
      // Disable menu item(s):
      listItem.setEnabled( false );

      // Listen for ActionEvents generated by menu items:
      connectItem.addActionListener( this );
      listItem.addActionListener( this );
      exitItem.addActionListener( this );
    }
  
    // ActionEvent handler:
    public void actionPerformed( ActionEvent e ) {
      if ( e.getSource() == connectItem ) {
        uname = new StringBuffer();
        
        InputDialog dialog = 
          new InputDialog(ChatWindow.this, "User Name:", uname);

        dialog.show();
        
        if (uname.length() != 0) {
          out.println("/n "+uname.toString());

          client.showStatus("connected as: "+uname.toString());
	
          cMenu.connectItem.setEnabled(false);
          cMenu.listItem.setEnabled(true);
          
          rMenu.gotoItem.setEnabled(true);
          rMenu.listItem.setEnabled(true);
          rMenu.exitItem.setEnabled(true);
        }
      } 
      else if ( e.getSource() == listItem ) {
	out.println("/u");
	client.showStatus("list users.");
      }
      else if ( e.getSource() == exitItem ) {
        setVisible( false );
        dispose();
        if ( --num_windows == 0 ) {
          System.exit(0);
        }
      }
    }
  }

  class RoomMenu extends Menu implements ActionListener {
    MenuItem gotoItem, listItem, exitItem;

    // Menu constructor:
    public RoomMenu() {
      // Invoke the constructor of the superclass:
      super( "Room" );

      gotoItem  
        = new MenuItem( "Go to ...", 
                        new MenuShortcut( KeyEvent.VK_G ) );
      listItem 
        = new MenuItem( "List rooms", 
                        new MenuShortcut( KeyEvent.VK_R ) );
      exitItem           
        = new MenuItem( "Return to lobby", 
                        new MenuShortcut( KeyEvent.VK_L ) );

      // Build the menu:
      add( gotoItem );
      add( listItem );
      add( exitItem );
      
      // Disable menu item(s):
      gotoItem.setEnabled( false );
      listItem.setEnabled( false );
      exitItem.setEnabled( false );

      // Listen for ActionEvents generated by menu items:
      gotoItem.addActionListener( this );
      listItem.addActionListener( this );
      exitItem.addActionListener( this );
    }
  
    // ActionEvent handler:
    public void actionPerformed( ActionEvent e ) {
      if ( e.getSource() == gotoItem ) {
        rname = new StringBuffer();
        
        InputDialog dialog = 
          new InputDialog(ChatWindow.this, "Room Name:", rname);

        dialog.show();

        if (rname.length() != 0) {
          out.println("/g "+rname.toString());
          client.showStatus("go to room: "+uname.toString());
        }
      } 
      else if ( e.getSource() == listItem ) {
	out.println("/r");
	client.showStatus("list rooms.");
      }
      else if ( e.getSource() == exitItem ) {
	out.println("/q");
      }
    }
  }

  class HelpMenu extends Menu implements ActionListener {
    MenuItem aboutItem;
    // Menu constructor:
    public HelpMenu() {
      // Invoke the constructor of the superclass:
      super( "Help" );
      aboutItem = new MenuItem("About Chat");      
      // Build the menu:
      add( aboutItem );
      aboutItem.addActionListener( this );
    }
  
    // ActionEvent handler:
    public void actionPerformed( ActionEvent e ) {
      if ( e.getSource() == aboutItem ) {
	out.println("/h");
      }      
    }
  }

  // query window
  class InputDialog extends Dialog implements ActionListener {
    TextField field = new TextField();
    Button button = new Button("Cancel");
    StringBuffer input;

    InputDialog (Frame parent, String msg, StringBuffer input) {
      super (parent, "Input", true);
      this.input = input;
      
      setSize(400, 80);
      //setResizable(false);

      add(new Panel(), BorderLayout.NORTH);
      add(new Panel(), BorderLayout.SOUTH);
      
      add(new Label(msg), BorderLayout.WEST);
      add(field, BorderLayout.CENTER);
      add(button, BorderLayout.EAST);

      field.addActionListener( this );
      button.addActionListener( this );

      client.showStatus("inputing data");
    }  

    // ActionEvent handler:
    public void actionPerformed(ActionEvent e) {
      if ( e.getSource() == field ) {
        input.append(e.getActionCommand());
	if (input.length()==0) 
          input.append("anonymous");
      } 
      else if (e.getSource() == button) {
      }

      setVisible(false);
      dispose();
    }
  } 

  public boolean action(Event e, Object what) {
    if (e.target == inputfield) {
      inputString = (String) e.arg;
      out.println((String)e.arg);
      inputfield.setText("");
      return true;
    }
    return false;
  }
}


// responds to an event generated by the close window button
class CloseWindow extends WindowAdapter {
  public void windowClosing (WindowEvent e) {
    e.getWindow().setVisible (false);
  }
}

// Wait for output from the server on the specified stream, and display
// it in the specified TextArea.
class StreamListener extends Thread {
  DataInputStream in;
  TextArea output;

  public StreamListener(DataInputStream in, TextArea output) {
    this.in = in;
    this.output = output;
    this.start();
  }

  public void run() {
    String line;

    try {
      for(;;) {
        line = in.readLine();
        if (line == null) 
	  break;
        output.appendText(line + "\n");
      }
    }
    catch (IOException e) { 
      output.appendText(e.toString()); 
    }
    finally { 
      output.setText("Connection lost."); 
    }
  }
}
