import java.net.*;
import java.io.*;
import java.util.*;

public class chatserver extends Thread {
    public final static int DEFAULT_PORT = 2222;
    protected int port;
    protected ServerSocket listen_socket;

    // Exit with an error message, when an exception occurs.
    public static void fail(Exception e, String msg) {
        System.err.println(msg + ": " +  e);
        System.exit(1);
    }
    
    // Create a ServerSocket to listen for connections on;  start the thread.
    public chatserver(int port) {
        if (port == 0) port = DEFAULT_PORT;
        this.port = port;
        try { listen_socket = new ServerSocket(port); }
        catch (IOException e) { fail(e, "Exception creating server socket"); }
        System.out.println("Server: listening on port " + port);
        this.start();
    }
    
    // The body of the server thread.  Loop forever, listening for and
    // accepting connections from clients.  For each connection, 
    // create a Connection object to handle communication through the
    // new Socket and add the connection object to a list, so that a server
	// can broadcast the message.
    public void run() {
        try {
            while(true) {
                Socket client_socket = listen_socket.accept();
                Connection c = new Connection(client_socket);
            }
        }
        catch (IOException e) { 
            fail(e, "Exception while listening for connections");
        }
    }
    
    // Start the server up, listening on an optionally specified port
    public static void main(String[] args) {
        int port = 0;
        if (args.length == 1) {
            try { port = Integer.parseInt(args[0]);  }
            catch (NumberFormatException e) { port = 0; }
        }
        new chatserver(port);
    }
}

// This class is the thread that handles all communication with a client
class Connection extends Thread {
	protected static Hashtable chatters = new Hashtable();
	protected static Vector rooms = new Vector();
	protected static int MAINROOM;

	/* The above variables are class variables,
	 * All of them are shared among all the threads. */

    protected int roomNum;
		// indicate this talker is in which room, default in main room.

    protected String line;

    protected Socket client; // socket to client.
    protected String chatterName;   // The chatter's name
    protected DataInputStream in;
    protected PrintStream out;

	public static void init() {
		rooms.addElement("mainroom");
		MAINROOM = rooms.indexOf("mainroom"); 
	}

    // Initialize the streams and start the thread
    public Connection(Socket client_socket) {
        client = client_socket;
		this.roomNum = MAINROOM;

        try { 
            in = new DataInputStream(client.getInputStream());
            out = new PrintStream(client.getOutputStream());
        }
        catch (IOException e) {
            try { client.close(); } catch (IOException e2) { ; }
            System.err.println("Exception while getting socket streams: " + e);
            return;
        }

        this.start();
    }

   // Provide the service.
    // Read a line, reverse it, send it back.  
    public void run() {
		try {
			do { 
				out.println("Please input /n nickname to enter the chatroom.");
				line = new String(in.readLine());
			} while (!line.startsWith("/n ")); 
		} catch (IOException e) { ; }

		this.out.println("Welcome to the chat room!");

		chatterName = (String) line.substring(3);
		chatters.put(this,chatterName);
		gotoRoom("mainroom");

        try {
            while(true) {
                // read in a line
                line = in.readLine();
                if (line == null) break;
                // and broadcast the chatter's name and the line he/she typed in
					clientActions(line);
            }
        }
        catch (IOException e) { ; }
        finally { try {client.close();} catch (IOException e2) {;} }
    }

	// Broadcast a piece of messege to a group of chatter(in the same room)
	private void brdcst(String mesg, int grpid) {
		Connection clnt;
		for (Enumeration e = chatters.keys(); e.hasMoreElements(); ) {
			clnt = (Connection) e.nextElement();
			if (clnt.roomNum == grpid)
				clnt.out.println(mesg);
		}
	}

	// To tell if the input is a normal string or a special action
	private boolean inputAnAction(String fromClient) {
		return(fromClient.startsWith("/")? true: false);	
	}

	/* To do the corresponding action if chatter input an action command
	 * or just broadcast the messege typed in */
	private void clientActions(String fromClient) {
		if(inputAnAction(fromClient))
			switch(fromClient.charAt(1)) {
				case 'g' : // goto another room is requested
					gotoRoom(fromClient.substring(3));
					break;
				case 'h' : // goto another room is requested
					listHelp();
					break;
				case 'i' : // invite another person to this room requested
					invite(fromClient.substring(3));
					break;
				case 'm' : // exit the room
					sendmesg(fromClient.substring(3));
					break;
				/* case 'n' : // action of changing a name is requested
					break; */
				case 'r' : // list rooms request
					listRooms();
					break;
				case 'u' : // list users in this room requested
					listGuests();
					break;
				case 'q' : // exit the room
					exitRoom();
					break;
				default :
					return;
			}
		else 
			brdcst(chatterName + ": " + line, this.roomNum);

		return;
	}

	// If the room specified by argument "roomName" does not
	// exist, ceate it and enter, otherwise, enter this room.
	private void gotoRoom(String roomName){
		int roomId;
		int oldRmNum;

 		roomId = rooms.indexOf(roomName);

		if (roomId == -1) 
			rooms.addElement(roomName);
		else
			brdcst(chatterName + "entered this room", roomId);

		oldRmNum = this.roomNum;
		this.roomNum = rooms.indexOf(roomName);
		if(!roomName.equals("mainroom")) 
			brdcst(chatterName + " left this room and entered room " + roomName, oldRmNum);
		this.out.println("You entered room " + roomName);
		listGuests();
	}
 
	private void sendmesg(String mesg){
		StringTokenizer st;
		Connection mesgrcver ;
		String userName;

		st = new StringTokenizer(mesg);
		if (st.hasMoreTokens()) {
			userName = st.nextToken();
			if(chatters.contains(userName)) {
				mesgrcver = findUser(userName);
				mesgrcver.out.println(this.chatterName + " sent you a message: ");
				mesgrcver.out.print("<<");
				while(st.hasMoreTokens()) {
					mesgrcver.out.print(" " + st.nextToken());
				}
				mesgrcver.out.print(" >>");
				mesgrcver.out.println();
			} else
				this.out.println("This user is not in this room");
		} else {
			this.out.println("Please use \"/m username mesg\" to send message");
		}
	}

	private void invite(String mesg){
		StringTokenizer st;
		Connection mesgrcver ;
		String userName;

		st = new StringTokenizer(mesg);
		if (st.hasMoreTokens()) {
			userName = st.nextToken();
			if(chatters.contains(userName)) {
				mesgrcver = findUser(userName);
				mesgrcver.out.println(this.chatterName + 
					" invited you to come to room " + rooms.elementAt(this.roomNum));
			} else
				this.out.println("This user is not in this room");
		} else {
			this.out.println("Please use \"/i username\" to invite a user to your room");
		}
	}

	private Connection findUser(String uname) {
		Connection clnt = this;

		for (Enumeration u = chatters.keys(); u.hasMoreElements(); ) {
			clnt = (Connection) u.nextElement();
			if (clnt.chatterName.equals(uname))
				break;
		}
		return clnt;
	}

	// List all the active chatroom besides main room
	private void listRooms() {
		this.out.println("Following rooms are in use");
		this.out.println("--------------------");
		for (Enumeration e = rooms.elements(); e.hasMoreElements();) {
			String roomName = (String) e.nextElement();	
			this.out.println(roomName);
		}
		this.out.println("-----End of Room List ----");
		this.out.println("You are now in room " + rooms.elementAt(this.roomNum));
	}

	// List the chatters in the current room I am in
	private void listGuests() {
		Connection clnt;

		if (!(chatters.size() == 1)) {
			this.out.println("Following users are chatting in this chatroom:");
			this.out.println("-----------------------------------");

			for (Enumeration e = chatters.keys(); e.hasMoreElements(); ) {
				clnt = (Connection) e.nextElement();
				if (clnt.roomNum == this.roomNum)
				this.out.println((String) chatters.get(clnt));
			}
			this.out.println("-------- End of user list -----------");
			this.out.println();
		} else {
			this.out.println("You are the first person in this chat room. You look lonely.");
		}
	}

	private void exitRoom() {
		int i = 0;
		Connection clnt;
		int oldRmNum = this.roomNum;

		if (this.roomNum == MAINROOM) {
			chatters.remove(this);
			if(!chatters.isEmpty())
				brdcst(this.chatterName + " said good bye to you and left this room",MAINROOM);
           	try { client.close(); } catch (IOException e2) { ; }
		}
		else {
			brdcst(chatterName + " came beck to this room.", MAINROOM);

			this.roomNum = MAINROOM; // Back to main room

			for (Enumeration e = chatters.keys(); e.hasMoreElements(); ) {
				clnt = (Connection) e.nextElement();
				if (clnt.roomNum == oldRmNum)
					i++;
			}

			if(i == 0)
				rooms.removeElementAt(roomNum);
			else
				brdcst(chatterName + " left this room.", oldRmNum);
		}
		return;
	}

	private void listHelp() {
		this.out.println("type \"/h\" to get this help information");
		this.out.println("type \"/g roomname\" to go to another room");
		this.out.println("type \"/i username\" to invite a user to your room");
		this.out.println("type \"/m username mesg\" to send a message to a user");
		this.out.println("type \"/r\" to list all the users in the current chat room");
		this.out.println("type \"/u\" to list all the users");
		this.out.println("type \"/q\" to quit current chat room");
	}
}
